/**
 * @name laravel-elixir-jsdoc
 * @author Shane Hendrix
 *
 * @desc Generate documentation for javascript files
 *
 * {@link https://github.com/jsdoc3/jsdoc link}
 *
 * @requires jsdoc
 */

/*global require */

var path = require('path');
var gulp   = require('gulp'),
    Elixir = require('laravel-elixir');

//noinspection JSUnresolvedFunction
var notify = new Elixir.Notification();

// REQUIRE:     File system functions
const fs = require('fs');

/*
 |----------------------------------------------------------------
 | JSDOC
 |----------------------------------------------------------------
 |
 | This task triggers the JSDOC software and shows
 | notifications depending on the success or the
 | failure of the JSDOC via elixir software
 |
 */

//noinspection JSUnresolvedFunction
Elixir.extend('jsdoc', function initJSDOC (src, command) {
    'use strict';

    var name = 'JSDOC';

    if (!command) {
        //noinspection JSUnresolvedFunction
        var bin  = path.normalize('node_modules/.bin/jsdoc'),
            dest = './storage/reports/jsdoc';

        //noinspection JSUnresolvedVariable,JSUnresolvedFunction
        src = src || Elixir.config.get('assets.js.folder');


        // Auto-detect config file
        try {
            let configFile = 'jsdoc.json';
            fs.statSync(configFile);

            // Update BIN if detected
            bin = `${bin} -c ${configFile}`;
        } catch (e) {
            return null;
        }


        command = bin
            + ' --recurse ' + src
            + ' --destination ' + dest;
    }

    //noinspection JSUnresolvedVariable,JSValidateTypes
    Elixir.Log.heading(`Triggering ${name}: ${command}`);


    //noinspection JSUnresolvedFunction,JSUnresolvedVariable
    return (
        gulp
            .src('')
            .pipe(Elixir.Plugins.shell(command))
            .on('error', function onError (e) {
                //noinspection JSUnresolvedFunction
                notify.error(e, `${name} error!`);

                //noinspection JSUnresolvedFunction
                this.emit('end');
            })
            .pipe(new Elixir.Notification(`${name} completed successfully`))
    );
});
